/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CAENCFDCONTROLLER_H
#define CAENCFDCONTROLLER_H

#include "CAENAdcController.h"
#include "drivers/ioxos/cfd/CfdDriver.h"

namespace npp {

/*!
 * \class CAENV1724Driver
 * \brief Main class for the CAENV1724Driver device driver
 *
 * \par
 * Class for CAENV1724Driver driver. Implements DeviceDriver methods.
 * \par
 * The CAENV1724Driver device manages the global functionality of the CAEN V1724 VME Digitiser.
 */
class CAENCfdController: public CAENAdcController {

public:

	//! Driver type value
	static const std::string TYPE;

	/*!
	 * \brief Constructor
	 * \param[in] name the name of the device driver
	 */
	CAENCfdController(const std::string& name);

	/*!
	 * \brief Destructor
	 */
	virtual ~CAENCfdController();

	/**
	 * \brief Postconfiguration
	 */
	virtual void postConfiguration();

	// Global properties
	Property<int32> delay;
	Property<int32> attenuation;
	// Channel property
	Property<int32> trapezoidRise;
	Property<int32> trapezoidFlat;
	Property<int32> decayTime;
	Property<int32> peaking;
	Property<int32> bitShift;
	Property<int32> channelOffset;

protected:

	DriverPtr<cfd::CfdDriver> m_CfdDriver;

private:

	void refreshDelayProperty(int32 aValue) throw (CannotSetValue);
	void refreshAttenuationProperty(int32 aValue) throw (CannotSetValue);
	void refreshTrapezoidRiseProperty(int32 aValue) throw (CannotSetValue);
	void refreshTrapezoidFlatProperty(int32 aValue) throw (CannotSetValue);
	void refreshDecayTimeProperty(int32 aValue) throw (CannotSetValue);
	void refreshPeakingProperty(int32 aValue) throw (CannotSetValue);
	void refreshBitShiftProperty(int32 aValue) throw (CannotSetValue);
	virtual void refreshChannelActiveProperty(int32 aValue) throw (CannotSetValue);
	virtual void refreshRecordLengthProperty(int32 aValue) throw (CannotSetValue);
	virtual void refreshDisplayChannelProperty(int32 aValue) throw (CannotSetValue);
	virtual void refreshChannelOffsetProperty(int32 aValue) throw (CannotSetValue);
	virtual void refreshNbChannelsProperty(int32 aValue) throw (CannotSetValue);
	virtual void refreshPulsePolarityProperty(int32 aValue) throw (CannotSetValue);
	virtual void refreshDppAcqModeProperty(int32 aValue) throw (CannotSetValue);
	virtual void updateBoardProperties();

	void updateCfdProperties(int32 channel);

	virtual void clearData();
	virtual void writeParam();
	virtual void initChannel(int32 aChannel);
};

}
#endif //CAENV1724CONTROLLER_H
